function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, llambda_est, param_est] = ...
    tobit_cre(Y,X,draws,param0,save_file)
warn2err = warning('error','MATLAB:nearlySingularMatrix');
warn2err2 = warning('error','MATLAB:illConditionedMatrix');

% save_file{1} nonempty: save MCMC details
    % save_file{3} = true: save draws needed for fcst only
    % save_file{3} = false: save draws needed for both fcst and 
    %                       recovering underlying dist
% save_file{1} empty: not save MCMC details
% save_file{2}: sample and estimator

%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
XX = X(2:end-1,:,:);
n_coef = size(XX,3)+1;
Xs = nan(T,N,n_coef);
if ~isempty(X)
    Xs(:,:,2:n_coef) = XX;
    X0 = [ones(N,1) permute(XX(1,:,:),[2,3,1])];
else
    X0 = ones(N,1);
end

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Prepare: prior parameters 
% rho, sigma2
M_coef0 = zeros(n_coef,1);
V_coef0 = ttau(1)*eye(n_coef);
a_ssigma20 = 3;
b_ssigma20 = mean(var(YY,1))*(a_ssigma20-1);

% K
if strcmp(param0{1},'normal')
    K = 1;
elseif strcmp(param0{1},'flexible')
    K = 20;
end
    
% p(lambda_i,y*_i0)
n_y = 2;
Phi0 = zeros(n_coef,n_y);
W0 = ttau(2)*eye(n_coef);
Psi0 = 4*diag([ones(1,n_y-1),ttau(3)]);
nu0 = 5+n_y;

if strcmp(param0{1},'flexible')
    a_aalpha0 = 2;
    b_aalpha0 = 2;
end

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho, sigma2
A = zeros(T-1,T);
for t = 1:T-1
    A(t,t) = sqrt((T-t)/(T-t+1));
    A(t,t+1:end) = -1/sqrt((T-t)*(T-t+1));
end
Xs(:,:,1) = YY(1:end-1,:);
AX = A*reshape(Xs,T,N*n_coef);
AY = A*YY(2:end,:);
vecAX = reshape(AX,N*(T-1),n_coef); 
rrho = (vecAX'*vecAX)\(vecAX'*AY(:));
ssigma2 = var(AY(:)-vecAX*rrho);

% lambda_i
llambda = mean(YY(2:end,:))-(reshape(mean(Xs),N,n_coef)*rrho)';

% p(lambda_i,y*_i0) 
if strcmp(param0{1},'normal')
	% Mean and variance
    aux_y = [llambda' YY_star(1,:)'];
    aux_x = X0;
    aux_xy = aux_x'*aux_y;
    aux_xx = aux_x'*aux_x;
    
    W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
    Phi1 = W1*aux_xy;
    Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
    nu1 = N+nu0;
    vv = iwishrnd((Psi1+Psi1')/2,nu1);
    mm = reshape(mvnrnd(Phi1(:),kron(vv,W1)),n_coef,n_y);
    
    gam = ones(1,N);
else
	% alpha
	aalpha = a_aalpha0/b_aalpha0;
	
	% gamma_i
	K0 = 10;
	gam = kmeans([llambda' YY_star(1,:)'],K0)';
	nk = zeros(1,K);
	for k = 1:K
		nk(k) = sum(gam==k);
	end
	
	% Draw p_k
	mb = cumsum(nk(K:-1:1));
	v = betarnd(1+nk(1:K-1),aalpha+mb(K-1:-1:1));
	pp = nan(1,K);
	rem = 1;
	for k = 1:K-1
		pp(k) = rem*v(k);
		rem = rem*(1-v(k));
	end
	pp(K) = rem;

	% Draw mu_k, omega2_k (need some randomness, o/w component variance can equal to 0)
    mm = nan(n_coef,n_y,K);
    vv = nan(n_y,n_y,K);
    for k = 1:K
        if sum(gam==k) > 0
            ixk = find(gam==k);
            aux_y = [llambda(ixk)' YY_star(1,ixk)'];
            aux_x = X0(ixk,:);
            aux_xy = aux_x'*aux_y;
            aux_xx = aux_x'*aux_x;
            
            W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
            Phi1 = W1*aux_xy;
            Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
            nu1 = nk(k)+nu0;
            vv(:,:,k) = iwishrnd((Psi1+Psi1')/2,nu1);
            mm(:,:,k) = reshape(mvnrnd(Phi1(:),kron(vv(:,:,k),W1)),n_coef,n_y);           
        else
            vv(:,:,k) = iwishrnd((Psi0+Psi0')/2,nu0);
            mm(:,:,k) = reshape(mvnrnd(Phi0(:),kron(vv(:,:,k),W0)),n_coef,n_y);
        end
    end
end

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef+1);
llambda_draw = nan(B_draw+N_draw,N);

if ~isempty(save_file{1}) 
    if ~save_file{3}
        YYT_star_draw = nan(B_draw+N_draw,N); 
    else
        YY_star_draw = nan(B_draw+N_draw,T1,N);
        mm_draw = nan(B_draw+N_draw,n_coef,n_y,K);
        vv_draw = nan(B_draw+N_draw,n_y,n_y,K);
        if strcmp(param0{1},'flexible')
            pp_draw = nan(B_draw+N_draw,K);
            aalpha_draw = nan(B_draw+N_draw,1);
        end
    end
end

%% Prepare: segment structure
seg_start = cell(1,N);
seg_end = cell(1,N);
n_seg = nan(1,N);
for i = 1:N
    seg_all = diff(YY(:,i)==0);
    seg_start{i} = find(seg_all==1)+1;
    seg_end{i} = find(seg_all==-1);
    if YY(1,i)==0
        seg_start{i} = [1; seg_start{i}];
    end
    if YY(end,i)==0
        seg_end{i} = [seg_end{i}; T1];
    end
    n_seg(i) = length(seg_start{i});
end

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw y*_it    
    DD_T1 = toeplitz(rrho(1).^(0:(T1-1)),[1 zeros(1,T1-1)]);
    
    QQ_T1 = diag((1-rrho(1).^((1:T1)*2))/(1-rrho(1)^2));
    for t = 1:T1
        QQ_T1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1(t,t);
        QQ_T1((t+1):T1,t) = QQ_T1(t,(t+1):T1);
    end
    
    for i = 1:N
        if n_seg(i) == 0
            continue
        end
        for i_seg = 1:n_seg(i)
            
            if seg_end{i}(i_seg) == T1
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+1;
            else
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+2;
            end
            
            if seg_start{i}(i_seg) == 1 % for y_i0 = 0
                aux_coeflambda = vv(1,2,gam(i))./vv(1,1,gam(i));
                mm_y0cond_x0 = mm(:,2,gam(i))-aux_coeflambda*mm(:,1,gam(i));
                if ~isempty(X)
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^((1:TT)-1)'*(X0(i,:)*mm_y0cond_x0+aux_coeflambda*llambda(i)) ...
                        +DD_T1(1:TT,1:TT)*[0; reshape(XX(seg_start{i}(i_seg)-1+(1:(TT-1)),i,:), ...
                                                      TT-1,n_coef-1)*rrho(2:end)];
                else
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^((1:TT)-1)'*(X0(i,:)*mm_y0cond_x0+aux_coeflambda*llambda(i));
                end
                
                QQ = diag([0 (1-rrho(1).^((1:(TT-1))*2))/(1-rrho(1)^2)] ...
                    +(vv(2,2,gam(i))-vv(1,2,gam(i))^2/vv(1,1,gam(i))) ...
                    /ssigma2*rrho(1).^((0:(TT-1))*2));
                for t = 1:TT
                    QQ(t,(t+1):TT) = rrho(1).^(1:(TT-t))*QQ(t,t);
                    QQ((t+1):TT,t) = QQ(t,(t+1):TT);
                end
            else
                if ~isempty(X)
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i) ...
                        +DD_T1(1:TT,1:TT)*reshape(XX(seg_start{i}(i_seg)-1+(0:(TT-1)),i,:), ...
                                                  TT,n_coef-1)*rrho(2:end);
                else
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i);
                end
                QQ = QQ_T1(1:TT,1:TT);
            end
            
            if seg_end{i}(i_seg) ~= T1
                AA = AA(1:(TT-1))+QQ(1:(TT-1),TT)/QQ(TT,TT)*(YY(seg_end{i}(i_seg)+1,i)-AA(TT));
                QQ = QQ(1:(TT-1),1:(TT-1))-QQ(1:(TT-1),TT)*QQ(TT,1:(TT-1))/QQ(TT,TT);
                TT = TT-1;
            end
            
            if TT == 1
                YY_star(seg_start{i}(i_seg),i) = AA+sqrt(QQ)*trandn(-inf,-AA/sqrt(QQ));
            else
                for t = 1:TT
                    aux_A = AA;
                    aux_A(t) = [];
                    aux_Ystar = YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i);
                    aux_Ystar(t) = [];
                    
                    aux_Q = QQ;
                    aux_Q(t,:) = [];
                    aux_v = aux_Q(:,t);
                    aux_Q(:,t) = [];
                    
                    aux_m = AA(t)+(aux_v'/aux_Q)*(aux_Ystar-aux_A);
                    aux_s = sqrt(QQ(t,t)-(aux_v'/aux_Q)*aux_v)*sqrt(ssigma2);
                    YY_star(seg_start{i}(i_seg)-1+t,i) = aux_m+aux_s*trandn(-inf,-aux_m/aux_s);
                end
            end
        end
    end
    
    %% Draw rho, sigma2 
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    Ys1 = Ys-llambda;
    vecX = reshape(Xs,N*T,n_coef);
    V_coef1 = eye(n_coef)/(vecX'*vecX/ssigma2+eye(n_coef)/V_coef0);
    M_coef1 = V_coef1*(vecX'*Ys1(:)/ssigma2+V_coef0\M_coef0);
    rrho = chol(V_coef1)'*randn(n_coef,1)+M_coef1;
    
    a_ssigma21 = a_ssigma20+N*T/2;
    b_ssigma21 = b_ssigma20+(Ys1(:)-vecX*rrho)'*(Ys1(:)-vecX*rrho)/2;
    ssigma2 = 1/gamrnd(a_ssigma21,1/b_ssigma21);
    
    %% Draw lambda_i
    zs = mean(Ys)-(reshape(mean(Xs),N,n_coef)*rrho)';
    if strcmp(param0{1},'normal')
        aux_coefy0 = vv(1,2)./vv(2,2);
        mm_lambdacond_x0 = mm(:,1)-aux_coefy0*mm(:,2);
        vv_lambdacond = vv(1,1)-vv(1,2)^2/vv(2,2);
        v_llambda1 = 1/(T/ssigma2+1/vv_lambdacond);
        m_llambda1 = ...
            (T*zs/ssigma2+((X0*mm_lambdacond_x0)'+aux_coefy0*YY_star(1,:))/vv_lambdacond)*v_llambda1;
        llambda = sqrt(v_llambda1)*randn(1,N)+m_llambda1;
    else
        aux_coefy0 = vv(1,2,:)./vv(2,2,:);
        mm_lambdacond_x0 = permute(mm(:,1,:)-aux_coefy0.*mm(:,2,:),[1 3 2]);
        mm_lambdacond_y0 = permute(aux_coefy0,[1 3 2]);
        vv_lambdacond = permute(vv(1,1,:)-vv(1,2,:).^2./vv(2,2,:),[1 3 2]);
        v_llambda1 = 1./(T/ssigma2+1./vv_lambdacond(gam));
        m_llambda1 = v_llambda1.*(zs*T/ssigma2+(sum(mm_lambdacond_x0(:,gam).*X0',1) ...
                                                +mm_lambdacond_y0(gam).*YY_star(1,:)) ...
                                               ./vv_lambdacond(gam));
        llambda = sqrt(v_llambda1).*randn(1,N)+m_llambda1;
    end
    
    %% Draw p(lambda_i,y*_i0) 
    if strcmp(param0{1},'normal')
        aux_y = [llambda' YY_star(1,:)'];
        aux_x = X0;
        aux_xy = aux_x'*aux_y;
        aux_xx = aux_x'*aux_x;
        
        W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
        Phi1 = W1*aux_xy;
        Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
        nu1 = N+nu0;
        vv = iwishrnd((Psi1+Psi1')/2,nu1);
        mm = reshape(mvnrnd(Phi1(:),kron(vv,W1)),n_coef,n_y);
    else
        % Draw alpha
        if abs(pp(K))>1e-6
            aalpha = gamrnd(a_aalpha0+K-1,1/(b_aalpha0-log(pp(K))));
        else
            u = betarnd(aalpha+1,N);
            v = rand(1);
            KK = sum(nk>0);
            if v < (a_aalpha0+KK-1)/N/(b_aalpha0-log(u))
                aalpha = gamrnd(a_aalpha0+KK,1/(b_aalpha0-log(u)));
            else
                aalpha = gamrnd(a_aalpha0+KK-1,1/(b_aalpha0-log(u)));
            end
        end
    
        % Draw p_k
        mb = cumsum(nk(K:-1:1));
        v = betarnd(1+nk(1:K-1),aalpha+mb(K-1:-1:1));
        pp = nan(1,K);
        rem = 1;
        for k = 1:K-1
            pp(k) = rem*v(k);
            rem = rem*(1-v(k));
        end
        pp(K) = rem;
        
        % Draw mu_k, omega2_k
        for k = 1:K
            if sum(gam==k) > 0
                ixk = find(gam==k);
                aux_y = [llambda(ixk)' YY_star(1,ixk)'];
                aux_x = X0(ixk,:);
                aux_xy = aux_x'*aux_y;
                aux_xx = aux_x'*aux_x;
                
                W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
                Phi1 = W1*aux_xy;
                Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
                nu1 = nk(k)+nu0;
                vv(:,:,k) = iwishrnd((Psi1+Psi1')/2,nu1);
                mm(:,:,k) = reshape(mvnrnd(Phi1(:),kron(vv(:,:,k),W1)),n_coef,n_y);
            else
                vv(:,:,k) = iwishrnd((Psi0+Psi0')/2,nu0);
                mm(:,:,k) = reshape(mvnrnd(Phi0(:),kron(vv(:,:,k),W0)),n_coef,n_y);
            end
        end
    
        % Draw gamma_i
        likel = nan(N,K);
        for k = 1:K
            likel(:,k) = mvnpdf([llambda' YY_star(1,:)'],X0*mm(:,:,k),vv(:,:,k));
        end
        P = pp.*likel;
        ixp = find(sum(P,2)==0);
        if ~isempty(ixp)
            P(ixp,1) = 1;
        end
        P = P./sum(P,2);
        ix_gam = mnrnd(1,P);
        [ixr,ixc] = find(ix_gam==1);
        gam = ixc(ixr)';
        for k = 1:K
            nk(k) = sum(gam==k);
        end
    end
    
    %% Prepare: forecast evaluation
    if ~isempty(X)
        m_draw(i_draw,:) = ([YY_star(end,:)' reshape(X(end,:,:),N,n_coef-1)]*rrho)'+llambda;
    else
        m_draw(i_draw,:) = (YY_star(end,:)'*rrho)'+llambda;
    end
    s_draw(i_draw,:) = sqrt(ssigma2);
        
    %% Save draws
    param_draw(i_draw,:) = [rrho' ssigma2];
    llambda_draw(i_draw,:) = llambda;    
    
    if ~isempty(save_file{1})
        if ~save_file{3}
            YYT_star_draw(i_draw,:) = YY_star(end,:);  
        else
            YY_star_draw(i_draw,:,:) = YY_star;
            mm_draw(i_draw,:,:,:) = mm;
            vv_draw(i_draw,:,:,:) = vv;
            if strcmp(param0{1},'flexible')
                pp_draw(i_draw,:) = pp;
                aalpha_draw(i_draw) = aalpha;
            end
        end        
    end
end

%% Forecast evaluation
fcst_eval;
    
%% Common and heterogeneous parameter estimates
llambda_est = mean(llambda_draw(ix_out,:));
param_aux = reshape([mean(param_draw(ix_out,:)); quantile(param_draw(ix_out,:),[.05 .95]')],1,[]);
param_est = [param_aux(1:end-3) nan(1,3) param_aux(end-2:end)];

%% Save MCMC details if needed
if ~isempty(save_file{1})     
    if ~save_file{3}
        parsave([save_file{1} save_file{2} '.mat'],param_draw,llambda_draw,YYT_star_draw)        
    else
        if strcmp(param0{1},'normal')
            parsave([save_file{1} save_file{2} '.mat'],param_draw,llambda_draw,YY_star_draw, ...
                mm_draw,vv_draw)
        else
            parsave([save_file{1} save_file{2} '.mat'],param_draw,llambda_draw,YY_star_draw, ...
                mm_draw,vv_draw,pp_draw,aalpha_draw)
        end
    end
end

warning(warn2err);
warning(warn2err2);