function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, het_est, param_est] = ...
    tobit_heterosk_cre(Y,X,draws,param0,save_file)
warn2err = warning('error','MATLAB:nearlySingularMatrix');
warn2err2 = warning('error','MATLAB:illConditionedMatrix');

% save_file{1} nonempty: save MCMC details
    % save_file{3} = true: save draws needed for fcst only
    % save_file{3} = false: save draws needed for both fcst and 
    %                       recovering underlying dist
% save_file{1} empty: not save MCMC details
% save_file{2}: sample and estimator

%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
XX = X(2:end-1,:,:);
n_coef = size(XX,3)+1;
Xs = nan(T,N,n_coef);
if ~isempty(X)
    Xs(:,:,2:n_coef) = XX;
    X0 = [ones(N,1) permute(XX(1,:,:),[2,3,1])];
else
    X0 = ones(N,1);
end

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Other specification
% Number of heterogeneous parameters
n_het = 2;

% For adaptive RWMH
c = 0.55;
ar_tg = 0.3;

%% Prepare: prior parameters 
% rho
M_coef0 = zeros(n_coef,1);
V_coef0 = ttau(1)*eye(n_coef);

% K
if strcmp(param0{1},'normal')
    K = 1;
elseif strcmp(param0{1},'flexible')
    K = 20;
end
    
% p(lambda_i,y*_i0), p(sigma2_i)
n_y = 2;
Phi0 = zeros(n_coef,n_y);
W0 = ttau(2)*eye(n_coef);
Psi0 = 4*diag([ones(1,n_y-1),ttau(3)]);
nu0 = 5+n_y;

m_mlnsigma2_0 = log(mean(var(YY,1)))-log(2)/2;
v_mlnsigma2_0 = 1;
a_vlnsigma2_0 = 3;
b_vlnsigma2_0 = log(2)*(a_vlnsigma2_0-1);

if strcmp(param0{1},'flexible')    
    a_aalpha0 = 2;
    b_aalpha0 = 2;
end

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho
A = zeros(T-1,T);
for t = 1:T-1
    A(t,t) = sqrt((T-t)/(T-t+1));
    A(t,t+1:end) = -1/sqrt((T-t)*(T-t+1));
end
Xs(:,:,1) = YY(1:end-1,:);
AX = A*reshape(Xs,T,N*n_coef);
AY = A*YY(2:end,:);
vecAX = reshape(AX,N*(T-1),n_coef); 
rrho = (vecAX'*vecAX)\(vecAX'*AY(:));

% lambda_i
llambda = mean(YY(2:end,:))-(reshape(mean(Xs),N,n_coef)*rrho)';

% sigma2_i
ssigma2 = var(reshape(AY(:)-vecAX*rrho,T-1,N));
ssigma2(ssigma2==0) = mean(ssigma2);

% p(lambda_i,y*_i0), p(sigma2_i)
if strcmp(param0{1},'normal')
    % Mean and variance
    aux_y = [llambda' YY_star(1,:)'];
    aux_x = X0;
    aux_xy = aux_x'*aux_y;
    aux_xx = aux_x'*aux_x;
    
    W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
    Phi1 = W1*aux_xy;
    Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
    nu1 = N+nu0;
    vv_lambday0 = iwishrnd((Psi1+Psi1')/2,nu1);
    mm_lambday0 = reshape(mvnrnd(Phi1(:),kron(vv_lambday0,W1)),n_coef,n_y);
    
    gam = ones(1,N);
    
    mm_lnsigma2 = mean(log(ssigma2));
    vv_lnsigma2 = var(log(ssigma2));
else
    % alpha
    aalpha = a_aalpha0/b_aalpha0*ones(n_het,1);
    pp = nan(n_het,K);
    nk = nan(n_het,K);
    gam = nan(n_het,N);
    K0 = 10;
    
    for i_het = 1:n_het       
        if i_het == 1
            aux_het = [llambda; YY_star(1,:)];
        else
            aux_het = log(ssigma2);
        end
        
        % gamma_i
        gam(i_het,:) = kmeans(aux_het',K0)';
        for k = 1:K
            nk(i_het,k) = sum(gam(i_het,:)==k);
        end
        
        % Draw p_k
        mb = cumsum(nk(i_het,K:-1:1));
        v = betarnd(1+nk(i_het,1:K-1),aalpha(i_het)+mb(K-1:-1:1));
        rem = 1;
        for k = 1:K-1
            pp(i_het,k) = rem*v(k);
            rem = rem*(1-v(k));
        end
        pp(i_het,K) = rem;
        
        % Draw mu_k, omega2_k (need some randomness, o/w component variance can equal to 0)
        if i_het == 1
            aux_het = [llambda; YY_star(1,:)];
            mm_lambday0 = nan(n_coef,n_y,K);
            vv_lambday0 = nan(n_y,n_y,K);
            for k = 1:K
                if sum(gam(i_het,:)==k) > 0
                    ixk = find(gam(i_het,:)==k);
                    aux_y = [llambda(ixk)' YY_star(1,ixk)'];
                    aux_x = X0(ixk,:);
                    aux_xy = aux_x'*aux_y;
                    aux_xx = aux_x'*aux_x;
                    
                    W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
                    Phi1 = W1*aux_xy;
                    Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
                    nu1 = nk(i_het,k)+nu0;
                    vv_lambday0(:,:,k) = iwishrnd((Psi1+Psi1')/2,nu1);
                    mm_lambday0(:,:,k) = ...
                        reshape(mvnrnd(Phi1(:),kron(vv_lambday0(:,:,k),W1)),n_coef,n_y);
                else
                    vv_lambday0(:,:,k) = iwishrnd((Psi0+Psi0')/2,nu0);
                    mm_lambday0(:,:,k) = ...
                        reshape(mvnrnd(Phi0(:),kron(vv_lambday0(:,:,k),W0)),n_coef,n_y);
                end
            end
        else
            aux_het = log(ssigma2);
            a_vlnsigma2_1 = a_vlnsigma2_0*ones(1,K);
            b_vlnsigma2_1 = b_vlnsigma2_0*ones(1,K);
            m_mlnsigma2_1 = m_mlnsigma2_0*ones(1,K);
            v_mlnsigma2_1 = v_mlnsigma2_0*ones(1,K);
            
            for k = 1:K
                if sum(gam(i_het,:)==k) > 0
                    ixk = find(gam(i_het,:)==k);
                    a_vlnsigma2_1(k) = a_vlnsigma2_0+nk(i_het,k)/2;
                    v_mlnsigma2_1(k) = 1/(1/v_mlnsigma2_0+nk(i_het,k));
                    m_mlnsigma2_1(k) = ...
                        v_mlnsigma2_1(k)*(m_mlnsigma2_0/v_mlnsigma2_0+sum(aux_het(ixk)));
                    b_vlnsigma2_1(k) = b_vlnsigma2_0+(m_mlnsigma2_0^2/v_mlnsigma2_0 ...
                        +sum(aux_het(ixk).^2)-m_mlnsigma2_1(k)^2/v_mlnsigma2_1(k))/2;
                end
            end
            vv_lnsigma2 = 1./gamrnd(a_vlnsigma2_1,1./b_vlnsigma2_1);
            mm_lnsigma2 = randn(1,K).*sqrt(v_mlnsigma2_1.*vv_lnsigma2)+m_mlnsigma2_1;
        end
    end   
end

% zeta for adaptive RWMH on sigma2_i
zzeta = ones(1,N);

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef);
het_draw = nan(B_draw+N_draw,N,n_het);

if ~isempty(save_file{1})
    if ~save_file{3}
        YYT_star_draw = nan(B_draw+N_draw,N);   
    else
        YY_star_draw = nan(B_draw+N_draw,T1,N);
        mm_lambday0_draw = nan(B_draw+N_draw,n_coef,n_y,K);
        vv_lambday0_draw = nan(B_draw+N_draw,n_y,n_y,K);
        mm_lnsigma2_draw = nan(B_draw+N_draw,K);
        vv_lnsigma2_draw = nan(B_draw+N_draw,K);
        if strcmp(param0{1},'flexible')
            aalpha_draw = nan(B_draw+N_draw,n_het);
            pp_draw = nan(B_draw+N_draw,K,n_het);
        end
    end
end

%% Prepare: segment structure
seg_start = cell(1,N);
seg_end = cell(1,N);
n_seg = nan(1,N);
for i = 1:N
    seg_all = diff(YY(:,i)==0);
    seg_start{i} = find(seg_all==1)+1;
    seg_end{i} = find(seg_all==-1);
    if YY(1,i)==0
        seg_start{i} = [1; seg_start{i}];
    end
    if YY(end,i)==0
        seg_end{i} = [seg_end{i}; T1];
    end
    n_seg(i) = length(seg_start{i});
end

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw y*_it
    DD_T1 = toeplitz(rrho(1).^(0:(T1-1)),[1 zeros(1,T1-1)]);
    
    QQ_T1 = diag((1-rrho(1).^((1:T1)*2))/(1-rrho(1)^2));
    for t = 1:T1
        QQ_T1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1(t,t);
        QQ_T1((t+1):T1,t) = QQ_T1(t,(t+1):T1);
    end
    
    for i = 1:N
        if n_seg(i) == 0
            continue
        end
        for i_seg = 1:n_seg(i)
            
            if seg_end{i}(i_seg) == T1
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+1;
            else
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+2;
            end
            
            if seg_start{i}(i_seg) == 1 % for y_i0 = 0
                aux_coeflambda = vv_lambday0(1,2,gam(1,i))./vv_lambday0(1,1,gam(1,i));
                mm_y0cond_x0 = mm_lambday0(:,2,gam(1,i))-aux_coeflambda*mm_lambday0(:,1,gam(1,i));
                if ~isempty(X)
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^((1:TT)-1)'*(X0(i,:)*mm_y0cond_x0+aux_coeflambda*llambda(i)) ...
                        +DD_T1(1:TT,1:TT)*[0; reshape(XX(seg_start{i}(i_seg)-1+(1:(TT-1)),i,:), ...
                                                      TT-1,n_coef-1)*rrho(2:end)];
                else
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^((1:TT)-1)'*(X0(i,:)*mm_y0cond_x0+aux_coeflambda*llambda(i));
                end
                
                QQ = diag([0 (1-rrho(1).^((1:(TT-1))*2))/(1-rrho(1)^2)] ...
                          +(vv_lambday0(2,2,gam(1,i)) ...
                            -vv_lambday0(1,2,gam(1,i))^2/vv_lambday0(1,1,gam(1,i))) ...
                           /ssigma2(i)*rrho(1).^((0:(TT-1))*2));
                for t = 1:TT
                    QQ(t,(t+1):TT) = rrho(1).^(1:(TT-t))*QQ(t,t);
                    QQ((t+1):TT,t) = QQ(t,(t+1):TT);
                end
            else
                if ~isempty(X)
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i) ...
                        +DD_T1(1:TT,1:TT)*reshape(XX(seg_start{i}(i_seg)-1+(0:(TT-1)),i,:), ...
                                                  TT,n_coef-1)*rrho(2:end);
                else
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda(i) ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i);
                end
                QQ = QQ_T1(1:TT,1:TT);
            end
            
            if seg_end{i}(i_seg) ~= T1
                AA = AA(1:(TT-1))+QQ(1:(TT-1),TT)/QQ(TT,TT)*(YY(seg_end{i}(i_seg)+1,i)-AA(TT));
                QQ = QQ(1:(TT-1),1:(TT-1))-QQ(1:(TT-1),TT)*QQ(TT,1:(TT-1))/QQ(TT,TT);
                TT = TT-1;
            end
            
            if TT == 1
                YY_star(seg_start{i}(i_seg),i) = AA+sqrt(QQ)*trandn(-inf,-AA/sqrt(QQ));
            else
                for t = 1:TT
                    aux_A = AA;
                    aux_A(t) = [];
                    aux_Ystar = YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i);
                    aux_Ystar(t) = [];
                    
                    aux_Q = QQ;
                    aux_Q(t,:) = [];
                    aux_v = aux_Q(:,t);
                    aux_Q(:,t) = [];
                    
                    aux_m = AA(t)+(aux_v'/aux_Q)*(aux_Ystar-aux_A);
                    aux_s = sqrt(QQ(t,t)-(aux_v'/aux_Q)*aux_v)*sqrt(ssigma2(i));
                    YY_star(seg_start{i}(i_seg)-1+t,i) = aux_m+aux_s*trandn(-inf,-aux_m/aux_s);
                end
            end
        end
    end
    
    %% Draw rho
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    Ys1 = Ys-llambda;
    vecX = reshape(Xs,N*T,n_coef);
    
    s_fac = reshape(repmat(sqrt(ssigma2),T,1),[],1);
    AYs = Ys1(:)./s_fac;
    AXs = vecX./s_fac;
    V_coef1 = eye(n_coef)/(AXs'*AXs+eye(n_coef)/V_coef0);
    M_coef1 = V_coef1*(AXs'*AYs+V_coef0\M_coef0);
    rrho = chol(V_coef1)'*randn(n_coef,1)+M_coef1;
    
    %% Draw lambda_i, sigma2_i
    zs = mean(Ys)-(reshape(mean(Xs),N,n_coef)*rrho)';
    sumres2 = sum(reshape(Ys1(:)-vecX*rrho,T,N).^2);
    
    if strcmp(param0{1},'normal')
        aux_coefy0 = vv_lambday0(1,2)/vv_lambday0(2,2);
        mm_lambdacond_x0 = mm_lambday0(:,1)-aux_coefy0*mm_lambday0(:,2);
        mm_lambdacond = ((X0*mm_lambdacond_x0)'+aux_coefy0*YY_star(1,:));
        vv_lambdacond = vv_lambday0(1,1)-vv_lambday0(1,2)^2/vv_lambday0(2,2);
        v_llambda1 = 1./(T./ssigma2+1/vv_lambdacond);
        m_llambda1 = (T*zs./ssigma2+mm_lambdacond/vv_lambdacond).*v_llambda1;

        for i = 1:N
            ssigma2c = randn(1)*zzeta(i)+ssigma2(i);
            if ssigma2c>0
                ar_ratio = exp(-(T/2+1)*(log(ssigma2c)-log(ssigma2(i))) ...
                               -sumres2(i)/2*(1/ssigma2c-1/ssigma2(i)) ...
                               -(log(ssigma2c)^2-log(ssigma2(i))^2 ...
                                 -2*mm_lnsigma2*(log(ssigma2c)-log(ssigma2(i)))) ...
                                /2/vv_lnsigma2);
            else
                ar_ratio = 0;
            end
            ar = min(1,ar_ratio);
            u = rand(1);
            ssigma2(i) = ssigma2c*(u<=ar)+ssigma2(i)*(u>ar);
            zzeta(i) = min(max(exp(log(zzeta(i))+i_draw^(-c)*(ar-ar_tg)),1e-10),1e10);
        end        
    else
        aux_coefy0 = vv_lambday0(1,2,:)./vv_lambday0(2,2,:);
        mm_lambdacond_x0 = permute(mm_lambday0(:,1,:)-aux_coefy0.*mm_lambday0(:,2,:),[1 3 2]);
        mm_lambdacond_y0 = permute(aux_coefy0,[1 3 2]);
        mm_lambdacond = ...
            sum(mm_lambdacond_x0(:,gam(1,:)).*X0',1)+mm_lambdacond_y0(gam(1,:)).*YY_star(1,:);
        vv_lambdacond_aux = ...
            permute(vv_lambday0(1,1,:)-vv_lambday0(1,2,:).^2./vv_lambday0(2,2,:),[1 3 2]);
        vv_lambdacond = vv_lambdacond_aux(gam(1,:));
        v_llambda1 = 1./(T./ssigma2+1./vv_lambdacond);
        m_llambda1 = v_llambda1.*(zs*T./ssigma2+mm_lambdacond./vv_lambdacond);
               
        for i = 1:N
            ssigma2c = randn(1)*zzeta(i)+ssigma2(i);
            if ssigma2c>0
                ar_ratio = exp(-(T/2+1)*(log(ssigma2c)-log(ssigma2(i))) ...
                               -sumres2(i)/2*(1/ssigma2c-1/ssigma2(i)) ...
                               -(log(ssigma2c)^2-log(ssigma2(i))^2 ...
                                 -2*mm_lnsigma2(gam(2,i))*(log(ssigma2c)-log(ssigma2(i)))) ...
                                /2/vv_lnsigma2(gam(2,i)));
            else
                ar_ratio = 0;
            end
            ar = min(1,ar_ratio);
            u = rand(1);
            ssigma2(i) = ssigma2c*(u<=ar)+ssigma2(i)*(u>ar);
            zzeta(i) = min(max(exp(log(zzeta(i))+i_draw^(-c)*(ar-ar_tg)),1e-10),1e10);
        end        
    end
    llambda = sqrt(v_llambda1).*randn(1,N)+m_llambda1;
    
    %% Draw p(lambda_i,y*_i0), p(sigma2_i)
    if strcmp(param0{1},'normal')
        aux_y = [llambda' YY_star(1,:)'];
        aux_x = X0;
        aux_xy = aux_x'*aux_y;
        aux_xx = aux_x'*aux_x;
        
        W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
        Phi1 = W1*aux_xy;
        Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
        nu1 = N+nu0;
        vv_lambday0 = iwishrnd((Psi1+Psi1')/2,nu1);
        mm_lambday0 = reshape(mvnrnd(Phi1(:),kron(vv_lambday0,W1)),n_coef,n_y);
        
        a_vlnsigma2_1 = a_vlnsigma2_0+N/2;
        v_mlnsigma2_1 = 1/(1/v_mlnsigma2_0+N);
        m_mlnsigma2_1 = v_mlnsigma2_1*(m_mlnsigma2_0/v_mlnsigma2_0+sum(log(ssigma2)));
        b_vlnsigma2_1 = b_vlnsigma2_0+(m_mlnsigma2_0^2/v_mlnsigma2_0 ...
            +sum(log(ssigma2).^2)-m_mlnsigma2_1^2/v_mlnsigma2_1)/2;
        vv_lnsigma2 = 1/gamrnd(a_vlnsigma2_1,1/b_vlnsigma2_1);
        mm_lnsigma2 = randn(1)*sqrt(v_mlnsigma2_1*vv_lnsigma2)+m_mlnsigma2_1;
    else
        for i_het = 1:n_het
            % Draw alpha
            if abs(pp(i_het,K))>1e-6
                aalpha(i_het) = gamrnd(a_aalpha0+K-1,1/(b_aalpha0-log(pp(i_het,K))));
            else
                u = betarnd(aalpha(i_het)+1,N);
                v = rand(1);
                KK = sum(nk(i_het,:)>0);
                if v < (a_aalpha0+KK-1)/N/(b_aalpha0-log(u))
                    aalpha(i_het) = gamrnd(a_aalpha0+KK,1/(b_aalpha0-log(u)));
                else
                    aalpha(i_het) = gamrnd(a_aalpha0+KK-1,1/(b_aalpha0-log(u)));
                end
            end
            
            % Draw p_k
            mb = cumsum(nk(i_het,K:-1:1));
            v = betarnd(1+nk(i_het,1:K-1),aalpha(i_het)+mb(K-1:-1:1));
            rem = 1;
            for k = 1:K-1
                pp(i_het,k) = rem*v(k);
                rem = rem*(1-v(k));
            end
            pp(i_het,K) = rem;
            
            % Draw mu_k, omega2_k
            if i_het == 1
                aux_het = [llambda; YY_star(1,:)];
                mm_lambday0 = nan(n_coef,n_y,K);
                vv_lambday0 = nan(n_y,n_y,K);
                for k = 1:K
                    if sum(gam(i_het,:)==k) > 0
                        ixk = find(gam(i_het,:)==k);
                        aux_y = [llambda(ixk)' YY_star(1,ixk)'];
                        aux_x = X0(ixk,:);
                        aux_xy = aux_x'*aux_y;
                        aux_xx = aux_x'*aux_x;
                        
                        W1 = eye(n_coef)/(aux_xx+eye(n_coef)/ttau(2));
                        Phi1 = W1*aux_xy;
                        Psi1 = Psi0+aux_y'*aux_y-aux_xy'*W1*aux_xy;
                        nu1 = nk(i_het,k)+nu0;
                        vv_lambday0(:,:,k) = iwishrnd((Psi1+Psi1')/2,nu1);
                        mm_lambday0(:,:,k) = ...
                            reshape(mvnrnd(Phi1(:),kron(vv_lambday0(:,:,k),W1)),n_coef,n_y);
                    else
                        vv_lambday0(:,:,k) = iwishrnd((Psi0+Psi0')/2,nu0);
                        mm_lambday0(:,:,k) = ...
                            reshape(mvnrnd(Phi0(:),kron(vv_lambday0(:,:,k),W0)),n_coef,n_y);
                    end
                end
            else
                aux_het = log(ssigma2);
                a_vlnsigma2_1 = a_vlnsigma2_0*ones(1,K);
                b_vlnsigma2_1 = b_vlnsigma2_0*ones(1,K);
                m_mlnsigma2_1 = m_mlnsigma2_0*ones(1,K);
                v_mlnsigma2_1 = v_mlnsigma2_0*ones(1,K);
                
                for k = 1:K
                    if sum(gam(i_het,:)==k) > 0
                        ixk = find(gam(i_het,:)==k);
                        a_vlnsigma2_1(k) = a_vlnsigma2_0+nk(i_het,k)/2;
                        v_mlnsigma2_1(k) = 1/(1/v_mlnsigma2_0+nk(i_het,k));
                        m_mlnsigma2_1(k) = ...
                            v_mlnsigma2_1(k)*(m_mlnsigma2_0/v_mlnsigma2_0+sum(aux_het(ixk)));
                        b_vlnsigma2_1(k) = b_vlnsigma2_0+(m_mlnsigma2_0^2/v_mlnsigma2_0 ...
                            +sum(aux_het(ixk).^2)-m_mlnsigma2_1(k)^2/v_mlnsigma2_1(k))/2;
                    end
                end
                vv_lnsigma2 = 1./gamrnd(a_vlnsigma2_1,1./b_vlnsigma2_1);
                mm_lnsigma2 = randn(1,K).*sqrt(v_mlnsigma2_1.*vv_lnsigma2)+m_mlnsigma2_1;
            end
            
            % Draw gamma_i 
            if i_het == 1
                likel = nan(N,K);
                for k = 1:K
                    likel(:,k) = mvnpdf(aux_het',X0*mm_lambday0(:,:,k),vv_lambday0(:,:,k));
                end
                P = pp(i_het,:).*likel;
            else
                P = pp(i_het,:).*normpdf(aux_het',mm_lnsigma2,sqrt(vv_lnsigma2));
            end
            ixp = find(sum(P,2)==0);
            if ~isempty(ixp)
                P(ixp,1) = 1;
            end
            P = P./sum(P,2);
            ix_gam = mnrnd(1,P);
            [ixr,ixc] = find(ix_gam==1);
            gam(i_het,:) = ixc(ixr)';
            for k = 1:K
                nk(i_het,k) = sum(gam(i_het,:)==k);
            end
    
        end
    end
    
    %% Prepare: forecast evaluation
    if ~isempty(X)
        m_draw(i_draw,:) = ([YY_star(end,:)' reshape(X(end,:,:),N,n_coef-1)]*rrho)'+llambda;
    else
        m_draw(i_draw,:) = (YY_star(end,:)'*rrho)'+llambda;
    end
    s_draw(i_draw,:) = sqrt(ssigma2);
    
    %% Save draws
    param_draw(i_draw,:) = rrho';
    het_draw(i_draw,:,:) = [llambda' ssigma2'];
    
    if ~isempty(save_file{1})
        if ~save_file{3}
            YYT_star_draw(i_draw,:) = YY_star(end,:);
        else
            YY_star_draw(i_draw,:,:) = YY_star;
            mm_lambday0_draw(i_draw,:,:,:) = mm_lambday0;
            vv_lambday0_draw(i_draw,:,:,:) = vv_lambday0;
            mm_lnsigma2_draw(i_draw,:) = mm_lnsigma2;
            vv_lnsigma2_draw(i_draw,:) = vv_lnsigma2;
            if strcmp(param0{1},'flexible')
                aalpha_draw(i_draw,:) = aalpha;
                pp_draw(i_draw,:,:) = pp';
            end
        end        
    end
end

%% Forecast evaluation
fcst_eval;
    
%% Common and heterogeneous parameter estimates
het_est = squeeze(mean(het_draw(ix_out,:,:)))';
param_aux = reshape([mean(param_draw(ix_out,:)); quantile(param_draw(ix_out,:),[.05 .95]')],1,[]);
param_est = [param_aux nan(1,6)];

%% Save MCMC details if needed
if ~isempty(save_file{1})  
    if ~save_file{3}
        parsave([save_file{1} save_file{2} '.mat'],param_draw,het_draw,YYT_star_draw)
    else
        if strcmp(param0{1},'normal')
            parsave([save_file{1} save_file{2} '.mat'],param_draw,het_draw,YY_star_draw, ...
                mm_lambday0_draw,vv_lambday0_draw,mm_lnsigma2_draw,vv_lnsigma2_draw)
        else
            parsave([save_file{1} save_file{2} '.mat'],param_draw,het_draw,YY_star_draw, ...
                mm_lambday0_draw,vv_lambday0_draw,mm_lnsigma2_draw,vv_lnsigma2_draw, ...
                pp_draw,aalpha_draw)
        end
    end 
end

warning(warn2err);
warning(warn2err2);