%%  Forecasting with a Panel Tobit Model
%   Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%   Empirical Application: Bank Loan Charge-off rates
%   Generate output tables and graphs

clear all
close all
clc

addpath(genpath('../tools')); % genpath for MMreg in predictive check
addpath('outputs');
mkdir results

data_path = 'data/';
temp_path = 'temp/';
result_path = 'results/';  
ix_X = 1:2; % []: no covariates
            % 1:2: x_it = [D(ln HPI_{it-1}), D(UR_{it-1})]', baseline setup
            % 1:3: x_it = [D(ln HPI_{it-1}), D(UR_{it-1}), D(ln INC_{it-1})]' 
prior_tuning = 2; % choice of prior tuning parameters (see Section E.1)
                  % 1: initial choice
                  % 2: adjusted choice
spec = ['app_x' strrep(num2str(ix_X),' ','') '_prior' num2str(prior_tuning)];

%% Load data and overall results 
load([temp_path spec '.mat'])
n_smpl0 = length(v_smpl0);

%% Outputs
% Generate tables: parameter estimates and set/density forecast performance (Tables 5, 7 and 8) 
table_est_fcst;

% Generate tables: 
% - Summary Statistics for Bank Characteristics (Table A-3)
% - Regression of heterogeneous coefficients on bank characteristics (Table 6)
table_BankVar;

% Generate graphs:
% - Log Predictive Density Scores - All Samples (Figures 2 and A-4)
% - Set Forecasts - All Samples (Figure 9)
plot_FcstEval;

% Generate graphs: Charge-Off Predictive Tail Probabilities, Spatial Dimension (Figures 3 and A-5)
plot_HetMap; 

% Generate graphs: Heterogeneous Coefficient Estimates (Figures 4 and A-6)
plot_HetCoef;

% Generate graphs: 
% Heterogeneous coefficients and probability of high-lambda group vs bank characteristics (Figure 5) 
plot_BankVar;

% Generate graphs: Effects (Term I and II) of HPI and UR (Figures 6 and A-7)
plot_MargEffect;

% Generate graphs: Posterior Predictive Checks: 
% Cross-sectional Distribution of Sample Statistics (Figures 7 and A-8) 
plot_PredCheck;

% Generate graphs: set forecasts where banks are sorted by posterior mean forecasts (Figure 8)
plot_sfcst;

% Generate graphs: Prior and Posterior for CRE Distribution (Figure A-3)
% - Scatter plots for moments
% - Historgrams for nubmers of modes
plot_PrPost;

%% Clean up paths
rmpath('../tools');
rmpath('outputs');