% Generate graphs:
% - Log Predictive Density Scores - All Samples 
% - Set Forecasts - All Samples 

%% Setup style
lgd = [];
SFont = 16;
graph_size = [6 6];

% Highlight selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2
ix0_smpl = find(ismember(v_smpl,v_smpl0)); 
ix1_smpl = setdiff(1:n_smpl,ix0_smpl);
color_ix0 = {'b','r'};

%% 1. Plot graphs: Log Predictive Density Scores - All Samples
v_alt = [2 3 5];
n_alt = length(v_alt);
alt_names = {'Normal','Flexible','normal_v_flex'; 'RE','CRE','re_v_cre';
    'Homoskedastic','Heteroskedastic','homosk_v_heterosk'}; % column order: [xlabel,ylabel,filename]
i_est_ref = 9;

table_aux = cell2mat(dfcst_stat);
table_aux2 = table_aux(:,1:n_dfcst:end);
table_graph = table_aux2([v_alt 1],:)-table_aux2(i_est_ref,:);
lims = [min(table_graph,[],2) max(table_graph,[],2)];

%% Loop over pairs
for i_graph = 1:n_alt
    F1 = figure;
    
    hold on
    scatter(table_graph(i_graph,ix1_smpl),table_graph(end,ix1_smpl),25,'k');
    for i_smpl0 = 1:n_smpl0
        scatter(table_graph(i_graph,ix0_smpl(i_smpl0)),table_graph(end,ix0_smpl(i_smpl0)),50, ...
            color_ix0{i_smpl0},'LineWidth',2);
    end
    hold off    
    
    aux_lim = lims([i_graph end],:);
    xylim = [min(aux_lim(:)) max(aux_lim(:))];
    xlim(xylim)
    ylim(xylim)
    
    h = refline([1,0]);
    h.Color = 'k';

    xlabel(alt_names{i_graph,1},'FontSize',SFont,'FontWeight','bold')
    ylabel(alt_names{i_graph,2},'FontSize',SFont,'FontWeight','bold')
    
    % Save graph
    graph_tag = [result_path spec '_lps_' alt_names{i_graph,3}];   
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
end

%% 2. Plot graphs: 
% Set Forecasts: Targeting Pointwise vs Average Coverage - All Samples
i_est = 1; % Flexible CRE, heterosk.

n_sfcst_tgt = 2;
table_graph = nan(n_sfcst_tgt,2,n_smpl);
for i_sfcst_tgt = 1:n_sfcst_tgt
    if i_sfcst_tgt == 1
        sfcst_stat = sfcstPtwise_stat;
    else
        sfcst_stat = sfcstAvg_stat;
    end
    table_aux = reshape(cell2mat(sfcst_stat(i_est,:)),[n_sfcst,n_smpl]);
    table_graph(i_sfcst_tgt,:,:) = table_aux(1:2,:);
end

table_graph(:,1,:) = abs(table_graph(:,1,:)-.9);

%% Group samples into improvement categories
group_names = {'improved','ambiguous'};
n_group = length(group_names);

ix_group = cell(1,n_group);
len_improvement = permute(diff(table_graph(:,1,:))<0,[3 2 1]);
ix_group{1} = find(len_improvement);
ix_group{2} = setdiff(1:n_smpl,ix_group{1});

xxlim = [0 max(reshape(table_graph(:,1,:),[],1))];
y_aux = reshape(table_graph(:,2,:),[],1);
yylim = [min(y_aux) max(y_aux)];

for i_group = 1:n_group
    F1 = figure;
    
    hold on
    for i_smpl = 1:n_smpl
        if ismember(i_smpl,ix_group{i_group})
            patchline(table_graph(:,1,i_smpl),table_graph(:,2,i_smpl),'linestyle','-', ...
                'edgecolor',[.15 .15 .15],'edgealpha',0.5);
        end
    end
    
    ix1_group = intersect(ix1_smpl,ix_group{i_group});
    scatter(permute(table_graph(2,1,ix1_group),[3 2 1]), ...
        permute(table_graph(2,2,ix1_group),[3 2 1]),25,'k');
    for i_smpl0 = 1:n_smpl0
        if ismember(ix0_smpl(i_smpl0),ix_group{i_group})
            scatter(table_graph(1,1,ix0_smpl(i_smpl0)),table_graph(1,2,ix0_smpl(i_smpl0)),50, ...
                color_ix0{i_smpl0},'marker','+','LineWidth',2);
            scatter(table_graph(2,1,ix0_smpl(i_smpl0)),table_graph(2,2,ix0_smpl(i_smpl0)),50, ...
                color_ix0{i_smpl0},'LineWidth',2);
        end
    end
    hold off
    
    xlim(xxlim)
    ylim(yylim)
    
    xlabel('|Cover. Freq. - 0.9|','FontSize',SFont,'FontWeight','bold')
    ylabel('Ave. Len.','FontSize',SFont,'FontWeight','bold')
    
    % Save graph
    graph_tag = [result_path spec '_sfcst_ptwise_v_avg_' group_names{i_group}];
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
end

%% Close files
close all