% Generate graphs: Heterogeneous Coefficient Estimates
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Setup style
graph_size = [6 6];
lgd = [];
SFont = 16;

n_centers = 50;

het_names = {'lambda_normalized','logsigma_normalized'}; % tags in file names
het_print = {'$\lambda_i/(1-\rho)$', '$\log(\sigma_i/\sqrt{1-\rho^2})$'}; % labels in output graphs
n_het = length(het_names);

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i
    sub_smpl_i; % subroutine: prepare sample i
    
    %% Normalized heterogeneous coefficients    
    x_var = mean(est_draw.het_draw(ix_out,:,1)./(1-est_draw.param_draw(ix_out,1)));
    y_var = mean(log(est_draw.het_draw(ix_out,:,2)./(1-est_draw.param_draw(ix_out,1).^2)))/2;
    
    % Exclude a few extreme observations
    if i_smpl0 == 1
        ix = x_var<6;
    else
        ix = (x_var<30) & (y_var<4);
    end
    x_var = x_var(ix);
    y_var = y_var(ix);
    
    %% 1. Histograms   
    for i_het = 1:n_het       
        if i_het == 1
            het_aux = x_var;
        else
            het_aux = y_var;
        end
        
        [counts,centers] = hist(het_aux,n_centers);
        counts_normalized = counts/(N*(centers(2)-centers(1))); 
            % still normalized by the full smpl size: limit the x-axis, not discard observations
           
        xxlim = [min(het_aux) max(het_aux)];
        yylim = [0 max(counts_normalized)*1.05];
        
        F1 = figure;
        bar(centers,counts_normalized,'FaceColor',[.7 .7 .7],'LineStyle','none')
        
        xlim(xxlim)
        ylim(yylim)
        xlabel(het_print{i_het},'FontSize',SFont,'FontWeight','bold','Interpreter','latex')
        
        % Save graph
        graph_tag = [result_path spec_smpl '_hist_' het_names{i_het}];
        graph_out(F1,graph_tag,[],SFont,graph_size,lgd);      
    end  
    
    %% 2. Scatter plots
    F1 = figure;
    
    scatter(x_var,y_var,25,'b');
    xlim([min(x_var) max(x_var)]*1.05)
    ylim([min(y_var) max(y_var)]*1.05)
    xlabel(het_print{1},'FontSize',SFont,'FontWeight','bold','Interpreter','latex')
    ylabel(het_print{2},'FontSize',SFont,'FontWeight','bold','Interpreter','latex')
    
    % Setup graph
    graph_tag = [result_path spec_smpl '_het_scatter'];
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);     
end

%% Close files
close all

