% Generate graphs: set forecasts where banks are sorted by posterior mean forecasts
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Set forecast parameters
sig0 = 0.1;

thin = 1;
N_thin = N_draw/thin;
ix_thin = ix_out(1:thin:end);

is_stat_only = 0; % report the all segments

%% Setup style
graph_size = [6 4];
lgd = [];
SFont = 16;

n_sfcst_tgt = 2;
color_shade = [1 .7 .7; .7 .7 .7];

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i and forecast
    sub_smpl_i; % subroutine: prepare sample i
    sub_fcst_prep; % subroutine: "Mean" and "StdD" of the trunc normal pred dist for each MCMC draw
    
    %% Posterior mean forecasts
    EY1_draw = m_draw.*normcdf(m_draw./s_draw)+s_draw.*normpdf(m_draw./s_draw);
    EY1 = mean(EY1_draw(ix_out,:));
    
    %% Prepare: set forecast
    Y1_draw = max(randn(B_draw+N_draw,N).*s_draw+m_draw,0);
    PY1_0_draw = normcdf(-m_draw./s_draw);
    
    sub_sfcst_triple; % subroutine: simulate triple (y_{i,T+h}^(j), pi_i^(j), W_i^(j))
    
    SY1_all = cell(n_sfcst_tgt,1);
    
    %% Set forecasts: targeting average covergage probability
    sub_sfcstAvg; % subroutine: set forecasts targeting average covergage probability      
    
    %% Set forecasts: targeting pointwise covergage probability
    sub_sfcstPtwise; % subroutine: set forecasts targeting pointwise covergage probability
    
    %% Combine results of two types of set forecasts
    n_seg = nan(n_sfcst_tgt,1);
    for i_sfcst_tgt = 1:n_sfcst_tgt
        n_seg(i_sfcst_tgt) = size(SY1_all{i_sfcst_tgt},3);
    end
    
    if max(n_seg)>1 % NEED adjustment if multiple positive segments
        warning([smpl ': Multiple positive segments!'])
    end    
    
    SY1_mat = nan(2*n_sfcst_tgt,N,max(n_seg));
        % 1st dim order: ["avg" lower & upper bound, "ptwise" lower & upper bound]
        % possible 3rd dim: multiple positive intervals
    for i_sfcst_tgt = 1:n_sfcst_tgt
        SY1_mat((i_sfcst_tgt-1)*2+(1:2),:,1:n_seg(i_sfcst_tgt)) = SY1_all{i_sfcst_tgt};
    end
    
    %% Plot graph
    % Sort units by posterior mean forecasts
    var_all = [Y(end,:); EY1; SY1_mat(:,:,1)];
    [~,ix_EY1] = sort(EY1);
    var_all = var_all(:,ix_EY1);
    
    % Plot
    F1 = figure;
    
    for i_sfcst_tgt = n_sfcst_tgt:-1:1 % set forecasts, targeting avg cover. in the top layer    
        jbfill(1:N,var_all(2+i_sfcst_tgt*2,:),var_all(1+i_sfcst_tgt*2,:), ...
            color_shade(i_sfcst_tgt,:),color_shade(i_sfcst_tgt,:),1,0.5);
        h = yline(0); % {0} is always part of set forecasts
        h.Color = color_shade(i_sfcst_tgt,:);
    end
    
    hold on
    plot(var_all(2,:),'k'); % posterior mean forecasts
    scatter(1:N,var_all(1,:),1,'b','.'); % actuals
    hold off
    
    xlim([1401*(i_chargeoff==6)+101*(i_chargeoff==7) N]) 
        % trim left tail as all forecasts are close to 0 for these units
    ylim([0 15*(i_chargeoff==6)+50*(i_chargeoff==7)]) % trim extreme upper end
    
    % Save graph
    graph_tag = [result_path spec_smpl '_sfcst_sorted'];
    graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
end

%% Close files
close all