function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, het_est, param_est] = ...
    tobit_re(Y,draws,param0,save_file)
%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
n_coef = 1;

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Prepare: prior parameters 
% rho, sigma2
M_coef0 = zeros(n_coef,1);
V_coef0 = ttau*eye(n_coef);
a_ssigma20 = 3;
b_ssigma20 = mean(var(YY,1))*(a_ssigma20-1);

% K
if strcmp(param0{1},'normal')
    K = 1;
elseif strcmp(param0{1},'flexible')
    K = 20;
end

% p(y*_i0)
m_my0_0 = 0;
v_my0_0 = ttau;
a_vy0_0 = 3;
b_vy0_0 = 2;
    
% p(lambda_i)
m_mlamb0 = 0;
v_mlamb0 = ttau;
a_vlamb0 = 3;
b_vlamb0 = 2;

if strcmp(param0{1},'flexible') 
    a_aalpha0 = 2;
    b_aalpha0 = 2;
end

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho, sigma2
A = zeros(T-1,T);
for t = 1:T-1
    A(t,t) = sqrt((T-t)/(T-t+1));
    A(t,t+1:end) = -1/sqrt((T-t)*(T-t+1));
end
Xs(:,:,1) = YY(1:end-1,:);
AX = A*reshape(Xs,T,N*n_coef);
AY = A*YY(2:end,:);
vecAX = reshape(AX,N*(T-1),n_coef); 
rrho = (vecAX'*vecAX)\(vecAX'*AY(:));
ssigma2 = var(AY(:)-vecAX*rrho);

% lambda_i
llambda = mean(YY(2:end,:))-(reshape(mean(Xs),N,n_coef)*rrho)';

% p(lambda_i) 
if strcmp(param0{1},'normal')
	% Mean and variance
    mm = mean(llambda);
    vv = var(llambda);
    gam = ones(1,N);
else
	% alpha
	aalpha = a_aalpha0/b_aalpha0;
	
	% gamma_i
	K0 = 10;
	gam = kmeans(llambda',K0)';
	nk = zeros(1,K);
	for k = 1:K
		nk(k) = sum(gam==k);
	end
	
	% Draw p_k
	mb = cumsum(nk(K:-1:1));
	v = betarnd(1+nk(1:K-1),aalpha+mb(K-1:-1:1));
	pp = nan(1,K);
	rem = 1;
	for k = 1:K-1
		pp(k) = rem*v(k);
		rem = rem*(1-v(k));
	end
	pp(K) = rem;

	% Draw mu_k, omega2_k (need some randomness, o/w component variance can equal to 0)
    a_vlamb1 = a_vlamb0*ones(1,K);
    b_vlamb1 = b_vlamb0*ones(1,K);
    m_mlamb1 = m_mlamb0*ones(1,K);
    v_mlamb1 = v_mlamb0*ones(1,K);
    for k = 1:K
        if sum(gam==k) > 0
            ixk = find(gam==k);
            a_vlamb1(k) = a_vlamb0+nk(k)/2;
            v_mlamb1(k) = 1/(1/v_mlamb0+nk(k));
            m_mlamb1(k) = v_mlamb1(k)*(m_mlamb0/v_mlamb0+sum(llambda(ixk)));
            b_vlamb1(k) = b_vlamb0 ...
                +(m_mlamb0^2/v_mlamb0+sum(llambda(ixk).^2)-m_mlamb1(k)^2/v_mlamb1(k))/2;
        end
    end
    vv = 1./gamrnd(a_vlamb1,1./b_vlamb1);
    mm = randn(1,K).*sqrt(v_mlamb1.*vv)+m_mlamb1;
end

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef+1);
het_draw = nan(B_draw+N_draw,N);

if ~isempty(save_file{1})   
    YY_star_draw = nan(B_draw+N_draw,T1,N);
    mm_y0_draw = nan(B_draw+N_draw,1);
    vv_y0_draw = nan(B_draw+N_draw,1);
    mm_draw = nan(B_draw+N_draw,K);
    vv_draw = nan(B_draw+N_draw,K);
    if strcmp(param0{1},'flexible')
        pp_draw = nan(B_draw+N_draw,K);
        aalpha_draw = nan(B_draw+N_draw,1);
    end
end

%% Prepare: segment structure
seg_start = cell(1,N);
seg_end = cell(1,N);
n_seg = nan(1,N);
for i = 1:N
    seg_all = diff(YY(:,i)==0);
    seg_start{i} = find(seg_all==1)+1;
    seg_end{i} = find(seg_all==-1);
    if YY(1,i)==0
        seg_start{i} = [1; seg_start{i}];
    end
    if YY(end,i)==0
        seg_end{i} = [seg_end{i}; T1];
    end
    n_seg(i) = length(seg_start{i});
end

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw p(y*_i0)    
    v_my0_1 = 1/(N+1/v_my0_0);
    m_my0_1 = (sum(YY_star(1,:))+m_my0_0/v_my0_0)*v_my0_1;
    a_vy0_1 = a_vy0_0+N/2;
    b_vy0_1 = b_vy0_0+(sum(YY_star(1,:).^2)+m_my0_0^2/v_my0_0-m_my0_1^2/v_my0_1)/2;
    vv_y0 = 1/gamrnd(a_vy0_1,1/b_vy0_1);
    mm_y0 = sqrt(v_my0_1*vv_y0)*randn(1)+m_my0_1;
    
    %% Draw y*_it     
    QQ_T1 = diag((1-rrho(1).^((1:T1)*2))/(1-rrho(1)^2));
    for t = 1:T1
        QQ_T1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1(t,t);
        QQ_T1((t+1):T1,t) = QQ_T1(t,(t+1):T1);
    end
    
    QQ_T1_1 = diag([0 (1-rrho(1).^((1:(T1-1))*2))/(1-rrho(1)^2)] ...
        +vv_y0/ssigma2*rrho(1).^((0:(T1-1))*2)); % for y_i0 = 0
    for t = 1:T1
        QQ_T1_1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1_1(t,t);
        QQ_T1_1((t+1):T1,t) = QQ_T1_1(t,(t+1):T1);
    end
    
    for i = 1:N
        if n_seg(i) == 0
            continue
        end
        for i_seg = 1:n_seg(i)
            
            if seg_end{i}(i_seg) == T1
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+1;
            else
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+2;
            end
            
            if seg_start{i}(i_seg) == 1
                AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda(i)+rrho(1).^(0:(TT-1))'*mm_y0;
                QQ = QQ_T1_1(1:TT,1:TT);
            else
                AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda(i) ...
                    +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i);
                QQ = QQ_T1(1:TT,1:TT);
            end
            
            if seg_end{i}(i_seg) ~= T1
                AA = AA(1:(TT-1))+QQ(1:(TT-1),TT)/QQ(TT,TT)*(YY(seg_end{i}(i_seg)+1,i)-AA(TT));
                QQ = QQ(1:(TT-1),1:(TT-1))-QQ(1:(TT-1),TT)*QQ(TT,1:(TT-1))/QQ(TT,TT);
                TT = TT-1;
            end
            
            if TT == 1
                YY_star(seg_start{i}(i_seg),i) = AA+sqrt(QQ)*trandn(-inf,-AA/sqrt(QQ));
            else
                for t = 1:TT
                    aux_A = AA;
                    aux_A(t) = [];
                    aux_Ystar = YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i);
                    aux_Ystar(t) = [];
                    
                    aux_Q = QQ;
                    aux_Q(t,:) = [];
                    aux_v = aux_Q(:,t);
                    aux_Q(:,t) = [];
                    
                    try
                        aux_m = AA(t)+(aux_v'/aux_Q)*(aux_Ystar-aux_A);
                    catch ME
                        disp([save_file{2} ': i_draw = ' num2str(i_draw) ', i = ' num2str(i) ...
                            ', i_seg = ' num2str(i_seg) ', t = ' num2str(t)])
%                         disp(ME.message);
                        disp('YY_seg = ');
                        disp(YY(seg_start{i}(i_seg):seg_end{i}(i_seg),i)')
                        disp('YY_star_seg = ');
                        disp(YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i)')
                        disp('aux_Q = ');
                        disp(aux_Q)
                        disp('eig(aux_Q) = ');
                        disp(eig(aux_Q)');
                        rethrow(ME)
                    end
                    aux_s = sqrt(QQ(t,t)-(aux_v'/aux_Q)*aux_v)*sqrt(ssigma2);
                    YY_star(seg_start{i}(i_seg)-1+t,i) = aux_m+aux_s*trandn(-inf,-aux_m/aux_s);
                end
            end
        end
    end
    
    %% Draw rho, sigma2 
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    Ys1 = Ys-llambda;
    vecX = reshape(Xs,N*T,n_coef);
    V_coef1 = eye(n_coef)/(vecX'*vecX/ssigma2+eye(n_coef)/V_coef0);
    M_coef1 = V_coef1*(vecX'*Ys1(:)/ssigma2+V_coef0\M_coef0);
    rrho = chol(V_coef1)'*randn(n_coef,1)+M_coef1;
    
    a_ssigma21 = a_ssigma20+N*T/2;
    b_ssigma21 = b_ssigma20+(Ys1(:)-vecX*rrho)'*(Ys1(:)-vecX*rrho)/2;
    ssigma2 = 1/gamrnd(a_ssigma21,1/b_ssigma21);
    
    %% Draw lambda_i 
    zs = mean(Ys)-(reshape(mean(Xs),N,n_coef)*rrho)';
    
    if strcmp(param0{1},'normal')
        v_llambda1 = 1/(T/ssigma2+1/vv);
        m_llambda1 = (T*zs/ssigma2+mm/vv)*v_llambda1;
        llambda = sqrt(v_llambda1)*randn(1,N)+m_llambda1;
    else
        v_llambda1 = 1./(T/ssigma2+1./vv(gam));
        m_llambda1 = v_llambda1.*(zs*T/ssigma2+mm(gam)./vv(gam));
        llambda = sqrt(v_llambda1).*randn(1,N)+m_llambda1;
    end
    
    %% Draw p(lambda_i) 
    if strcmp(param0{1},'normal')
        a_vlamb1 = a_vlamb0+N/2;
        v_mlamb1 = 1/(1/v_mlamb0+N);
        m_mlamb1 = v_mlamb1*(m_mlamb0/v_mlamb0+sum(llambda));
        b_vlamb1 = b_vlamb0+(m_mlamb0^2/v_mlamb0+sum(llambda.^2)-m_mlamb1^2/v_mlamb1)/2;
        vv = 1/gamrnd(a_vlamb1,1/b_vlamb1);
        mm = randn(1)*sqrt(v_mlamb1*vv)+m_mlamb1;       
    else
        % Draw alpha
        if abs(pp(K))>1e-6
            aalpha = gamrnd(a_aalpha0+K-1,1/(b_aalpha0-log(pp(K))));
        else
            u = betarnd(aalpha+1,N);
            v = rand(1);
            KK = sum(nk>0);
            if v < (a_aalpha0+KK-1)/N/(b_aalpha0-log(u))
                aalpha = gamrnd(a_aalpha0+KK,1/(b_aalpha0-log(u)));
            else
                aalpha = gamrnd(a_aalpha0+KK-1,1/(b_aalpha0-log(u)));
            end
        end
    
        % Draw p_k
        mb = cumsum(nk(K:-1:1));
        v = betarnd(1+nk(1:K-1),aalpha+mb(K-1:-1:1));
        pp = nan(1,K);
        rem = 1;
        for k = 1:K-1
            pp(k) = rem*v(k);
            rem = rem*(1-v(k));
        end
        pp(K) = rem;
        
        % Draw mu_k, omega2_k
        a_vlamb1 = a_vlamb0*ones(1,K);
        b_vlamb1 = b_vlamb0*ones(1,K);
        m_mlamb1 = m_mlamb0*ones(1,K);
        v_mlamb1 = v_mlamb0*ones(1,K);
        for k = 1:K
            if sum(gam==k) > 0
                ixk = find(gam==k);
                a_vlamb1(k) = a_vlamb0+nk(k)/2;
                v_mlamb1(k) = 1/(1/v_mlamb0+nk(k));
                m_mlamb1(k) = v_mlamb1(k)*(m_mlamb0/v_mlamb0+sum(llambda(ixk)));
                b_vlamb1(k) = b_vlamb0 ...
                    +(m_mlamb0^2/v_mlamb0+sum(llambda(ixk).^2)-m_mlamb1(k)^2/v_mlamb1(k))/2;
            end
        end
        vv = 1./gamrnd(a_vlamb1,1./b_vlamb1);
        mm = randn(1,K).*sqrt(v_mlamb1.*vv)+m_mlamb1;
    end
    
    %% Draw gamma_i
    if K > 1
        P = pp.*normpdf(llambda',mm,sqrt(vv));
        ixp = find(sum(P,2)==0);
        if ~isempty(ixp)
            P(ixp,1) = 1;
        end
        P = P./sum(P,2);
        ix_gam = mnrnd(1,P);
        [ixr,ixc] = find(ix_gam==1);
        gam = ixc(ixr)';
        for k = 1:K
            nk(k) = sum(gam==k);
        end
    end
    
    %% Prepare: forecast evaluation
    m_draw(i_draw,:) = YY_star(end,:)*rrho+llambda;
    s_draw(i_draw,:) = sqrt(ssigma2);
    
    %% Save draws
    param_draw(i_draw,:) = [rrho' ssigma2];
    het_draw(i_draw,:) = llambda;   
    
    if ~isempty(save_file{1})
        YY_star_draw(i_draw,:,:) = YY_star;
        mm_y0_draw(i_draw,:) = mm_y0;
        vv_y0_draw(i_draw,:) = vv_y0;
        mm_draw(i_draw,:) = mm;
        vv_draw(i_draw,:) = vv;
        if strcmp(param0{1},'flexible')
            pp_draw(i_draw,:) = pp;
            aalpha_draw(i_draw) = aalpha;
        end
    end   
end

%% Forecast evaluation
fcst_eval;

%% Common and heterogeneous parameter estimates
het_est = mean(het_draw(ix_out,:));
param_aux = reshape([mean(param_draw(ix_out,:));
    quantile(param_draw(ix_out,:),[.05 .95])],1,[]);
param_est = [param_aux(1:end-3) nan(1,3) param_aux(end-2:end)];

%% Save MCMC details if needed
if ~isempty(save_file{1})
    if strcmp(param0{1},'normal')
        parsave([save_file{1} save_file{2} '.mat'],param_draw,het_draw,YY_star_draw, ...
            mm_y0_draw,vv_y0_draw,mm_draw,vv_draw)
    else
        parsave([save_file{1} save_file{2} '.mat'],param_draw,het_draw,YY_star_draw, ...
            mm_y0_draw,vv_y0_draw,mm_draw,vv_draw,pp_draw,aalpha_draw)
    end
end