% Generate graphs: Posterior Means and Estimated RE Distributions for lambda_i 

%% Setup style
het_names = {'lambda'};
i_het = 1;

n_centers = 50;

graph_size = [6 4];
SFont = 16;
lgd = [];

v_estout = 1:2;
n_estout = length(v_estout);
is_estout = [1 1 1; 1 0 0];

thin = 10; % use every 10th draw to produce the hairlines, change if needed

est_draw = cell(n_estout,n_dgp);
het_cell = cell(1,n_estout,n_dgp);

group_names = {'across_est','across_dgp'};
n_group = length(group_names);

%% Load data, overall results, and estimator-specific results
for i_dgp = 1:n_dgp
    spec = ['dgp' num2str(i_dgp)];
    load([temp_path spec '.mat']);
    
    % Posterior means of individual heterogeneities
    for i_estout = 1:n_estout
        if is_estout(i_estout,i_dgp)
            i_est = v_estout(i_estout);
            het_cell{1,i_estout,i_dgp} = het_est{i_est,1}(1,:)';
            est_draw{i_estout,i_dgp} = matfile([temp_path spec '_isim1_' est_names{i_est} '.mat']);
        else
            het_cell{1,i_estout,i_dgp} = nan(N,1);
        end
    end
end

%% Prepare for graphs
ix_out = B_draw+(1:N_draw);
het_mat = cell2mat(het_cell);
F1 = cell(n_estout,n_dgp);

%% Loop over sets of graphs
%  For easy comparison, all figures in each group have the same bin centers and x,y-scales.
for i_group = 1:n_group
    %% Prepare for graphs
    if i_group == 1 % first group of graphs: compare across estimators, DGP 1 
        the_is_estout = [ones(n_estout,1) zeros(n_estout,n_dgp-1)];
    else % second group of graphs: compare across DGPs, Flexible CRE heterosk. estimator
        the_is_estout = [is_estout(1,:); zeros(n_estout-1,n_dgp)];
    end
    
    the_v_dgp = find(sum(the_is_estout,1));
    the_v_estout = cell(1,n_dgp);
    
    [counts,centers] = hist(het_mat(:,the_is_estout==1),n_centers);
    counts_normalized = counts/(N*(centers(2)-centers(1)));
    xxlim = [min(het_mat(:)) max(het_mat(:))];
    ymax_dens = nan(1,n_dgp);
    
    %% Plot graphs
    for i_dgp = the_v_dgp
        % Load data and overall results
        spec = ['dgp' num2str(i_dgp)];
        load([temp_path spec '.mat']);
        
        % Calculate true density
        p_centers_true = sum(repmat(p_llambda,n_centers,1) ...
                             .*normpdf(repmat(centers,1,2),repmat(m_llambda,n_centers,1), ...
                                       repmat(sqrt(v_llambda),n_centers,1)),2);
        ymax_dens(1,i_dgp) = max(p_centers_true);
        
        the_v_estout{i_dgp} = find(the_is_estout(:,i_dgp))';
        for i_estout = the_v_estout{i_dgp}
            i_est = v_estout(i_estout);
            
            % Calculate posterior distribution of RE distribution of individual heterogeneities
            p_centers = nan(N_draw/thin,n_centers);
            if ismember(i_est,[1 3])
                for i_centers = 1:n_centers
                    p_centers(:,i_centers) = sum( ...
                        est_draw{i_estout,i_dgp}.pp_draw(ix_out(1:thin:N_draw),:,1) ...
                        .*normpdf(centers(i_centers), ...
                                  est_draw{i_estout,i_dgp}.mm_draw(ix_out(1:thin:N_draw),:), ...
                                  sqrt(est_draw{i_estout,i_dgp}.vv_draw(ix_out(1:thin:N_draw),:))),2);
                end
            elseif ismember(i_est,[2 4])
                for i_centers = 1:n_centers
                    p_centers(:,i_centers) = ...
                        normpdf(centers(i_centers), ...
                                est_draw{i_estout,i_dgp}.mm_draw(ix_out(1:thin:N_draw),:), ...
                                sqrt(est_draw{i_estout,i_dgp}.vv_draw(ix_out(1:thin:N_draw),:)));
                end
                
            end
            
            % Figure
            F1{i_estout,i_dgp} = figure;
            hold on
            
            % Plot histogram of posterior means for individual heterogeneities
            bar(centers,counts_normalized(:,sum(the_is_estout(1:(i_dgp-1)*n_estout+i_estout))),...
                'LineStyle','none','FaceColor',[.7 .7 .7])
            
            % Plot posterior distribution of RE distribution of individual heterogeneities
            for ii_draw = 1:N_draw/thin
                patchline(centers,p_centers(ii_draw,:)','linestyle','-','edgecolor','b', ...
                    'linewidth',2,'edgealpha',0.05);
            end
            
            % Plot true density
            plot(centers,p_centers_true,'k-','linewidth',2)
            hold off
        end
    end
    
    %% Set common x,y-scales and save graphs
    yylim = [0 max([counts_normalized(:); ymax_dens(:)])];
    for i_dgp = the_v_dgp
        spec = ['dgp' num2str(i_dgp)];
        
        for i_estout = the_v_estout{i_dgp}
            i_est = v_estout(i_estout);
            
            % Set common x,y-scales
            figure(F1{i_estout,i_dgp});
            xlim(xxlim);
            ylim(yylim);
            
            % Save graph
            graph_tag = [result_path spec '_' het_names{i_het} '_' est_names{i_est} ...
                '_' group_names{i_group}];
            graph_out(F1{i_estout,i_dgp},graph_tag,[],SFont,graph_size,lgd);
        end
    end
end

%% Close files
close all