% Generate tables: parameter estimates and set/density forecast performance

%% Setup style
est_print = ... % row labels in output tables
   {'Flexible \& Heterosk.';
    'Normal \& Heterosk.  ';
    'Flexible \& Homosk.  ';
    'Normal \& Homosk.    ';
    'Pooled Tobit         ';
    'Pooled Linear        '};

file_names = {'dfcst_sfcst_param'};
n_file = length(file_names);
file_id = nan(1,n_file);
for i_file = 1:n_file
    file_id(i_file) = fopen([result_path '' file_names{i_file} '.txt'],'w'); 
    % change to 'w'/'a' if needed
end
ncol_file = 8; % number of columns in each output table
    
%% Loop over DGPs
for i_dgp = 1:n_dgp
    %% Load data and overall results
    spec = ['dgp' num2str(i_dgp)];
    load([temp_path spec '.mat'])

    %% Prepare output tables
    table_sfcstPtwise = mean(reshape(cell2mat(sfcstPtwise_stat),[n_est,n_sfcst,n_sim]),3);
    table_sfcstAvg = mean(reshape(cell2mat(sfcstAvg_stat),[n_est,n_sfcst,n_sim]),3);
    table_dfcst = mean(reshape(cell2mat(dfcst_stat),[n_est,n_dfcst,n_sim]),3);
    param_aux = reshape(cell2mat(param_est),[n_est,n_param*3,n_sim]);
    table_param = mean(param_aux,3);
    
    table_out = {[table_dfcst table_sfcstAvg(:,1:2) table_sfcstPtwise(:,1:2) ... 
        table_param(:,1)-rrho std(param_aux(:,1,:),[],3)]};
    % order: [LPS and CRPS for density forecasts,
    %         Cover. Freq., Ave. Len. for set forecasts targeting average coverage,
    %         Cover. Freq., Ave. Len. for set forecasts targeting pointwise coverage,
    %         Bias and StdD in rho-hat]
    
    %% Generate output tables
    for i_file = 1:n_file
        fprintf(file_id(i_file),['DGP' num2str(i_dgp) '\r\n']);
        for i_est = 1:n_est
            fprintf(file_id(i_file), ...
                ['%4.30s ' repmat('& %7.3f ',1,ncol_file(i_file)) ' %4.6s\r\n'], ...
                est_print{i_est},table_out{i_file}(i_est,:),'\\ ');
        end
        fprintf(file_id(i_file),'\r\n');
    end
end

%% Close files
for i_file = 1:n_file
    fclose(file_id(i_file));
end